#!/bin/bash
input_file=$1 #"./test_input/test_sequence_I.fa"
tmp_folder=$2 #"./tmp/"
tmp_folder=$(mktemp -d -t ci-XXXXXXXXXX --tmpdir="$tmp_folder")"/"
model_folder=$3 #"./models/"
in_file_name=`basename "$input_file"`
target_file=$4$in_file_name #"./test_output/"
gkmpredict_location=$5 #"./bin/gkmpredict"

order_model_folder="${model_folder}Order/*"
family_model_folder="${model_folder}Family/"
genus_model_folder="${model_folder}Genus/"
model_suffix=".model.txt"
predict_suffix="_pred_tmp"
col_order="${tmp_folder}collected_order.txt"
col_fam="${tmp_folder}collected_family.txt"
col_genus="${tmp_folder}collected_genus.txt"


##declare lookup tables for taxonomy
declare -A order_to_family
declare -A family_to_genus

order_to_family[Methylococcales]="Methylococcaceae Crenotrichaceae"
order_to_family[Enterobacterales]="Enterobacteriaceae Erwiniaceae Yersiniaceae Hafniaceae Morganellaceae Pectobacteriaceae Budviciaceae"
order_to_family[Alteromonadales]="Alteromonadaceae Pseudoalteromonadaceae Idiomarinaceae Shewanellaceae Ferrimonadaceae Colwelliaceae Psychromonadaceae Moritellaceae"
order_to_family[Pseudomonadales]="Pseudomonadaceae Moraxellaceae Ventosimonadaceae"
order_to_family[Oceanospirillales]="Oceanospirillaceae Alcanivoracaceae Halomonadaceae Hahellaceae Kangiellaceae Oleiphilaceae Endozoicomonadaceae Saccharospirillaceae Balneatrichaceae"
order_to_family[Thiotrichales]="Piscirickettsiaceae Francisellaceae Thiotrichaceae  Thiolinaceae unclassified Thiotrichales Thiofilaceae Fastidiosibacteraceae"
order_to_family[Xanthomonadales]="Xanthomonadaceae Rhodanobacteraceae"
order_to_family[Cellvibrionales]="Porticoccaceae  Halieaceae Cellvibrionaceae Spongiibacteraceae Microbulbiferaceae"
order_to_family[Aeromonadales]="Aeromonadaceae Succinivibrionaceae"
order_to_family[Legionellales]="Coxiellaceae Legionellaceae"
order_to_family[CandidatusCompetibacteraceae]=""
order_to_family[Chromatiales]="Chromatiaceae Halothiobacillaceae Ectothiorhodospiraceae Granulosicoccaceae Wenzhouxiangellaceae"
order_to_family[Salinisphaerales]="Salinisphaeraceae"
order_to_family[Vibrionales]="Vibrionaceae"
order_to_family[Nevskiales]="Sinobacteraceae Algiphilaceae"
order_to_family[Pasteurellales]="Pasteurellaceae"
order_to_family[unclassified Gammaproteobacteria]=""
order_to_family[Cardiobacteriales]="Cardiobacteriaceae"
order_to_family[Ignatzschineria]=""
order_to_family[Orbales]="Orbaceae"
order_to_family[Thiolapillus]=""
order_to_family[Pseudohongiella]=""
order_to_family[Thiohalorhabdus]=""
order_to_family[Neiella]=""
order_to_family[Wohlfahrtiimonas]=""
order_to_family[Acidiferrobacterales]="Acidiferrobacteraceae"
order_to_family[Immundisolibacterales]="Immundisolibacteraceae"
order_to_family[unclassifiedGammaproteobacteria]=""

family_to_genus[Methylococcaceae]="Methylobacter  Methylococcus Methylomonas Methylomicrobium Methylogaea Methylosarcina Methyloterricola Methylovulum Methylomagnum Methylocaldum Methylomarinum Methyloprofundus"
family_to_genus[Enterobacteriaceae]="Leclercia Pluralibacter Enterobacter Klebsiella Cronobacter Siccibacter Buttiauxella Escherichia Citrobacter Salmonella Shigella Shimwellia Raoultella Kosakonia Kluyvera Trabulsiella Cedecea Rosenbergiella Lelliottia Plesiomonas Franconibacter Pseudescherichia Yokenella Atlantibacter Izhakiella Superficieibacter"
family_to_genus[Alteromonadaceae]="Alteromonas Marinobacter Glaciecola Paraglaciecola Aestuariibacter Alishewanella Tamilnaduibacter Catenovulum Aliagarivorans Gayadomonas Agarivorans Bowmanella Mangrovitalea"
family_to_genus[Pseudomonadaceae]="Pseudomonas  Oblitimonas Azotobacter Thiopseudomonas"
family_to_genus[Moraxellaceae]="Acinetobacter Psychrobacter  Moraxella Perlucidibaca Paraperlucidibaca"
family_to_genus[Oceanospirillaceae]="Thalassolituus  Neptuniibacter Marinomonas Oleispira Bacterioplanes Marinospirillum Oceanospirillum Nitrincola Neptunomonas Marinobacterium Motiliproteus"
family_to_genus[Piscirickettsiaceae]="Methylophaga Thiomicrospira  Cycloclasticus Hydrogenovibrio Piscirickettsia Thiomicrorhabdus Sulfurivirga"
family_to_genus[Xanthomonadaceae]="Stenotrophomonas  Xanthomonas Pseudoxanthomonas Xylella Lysobacter Arenimonas Luteimonas Thermomonas"
family_to_genus[Porticoccaceae]="Porticoccus"
family_to_genus[Pseudoalteromonadaceae]="Pseudoalteromonas Psychrosphaera"
family_to_genus[Francisellaceae]="Francisella Allofrancisella"
family_to_genus[Erwiniaceae]="Pantoea Tatumella Erwinia Buchnera Wigglesworthia  Mixta"
family_to_genus[Idiomarinaceae]="Idiomarina Aliidiomarina"
family_to_genus[Aeromonadaceae]="Aeromonas Oceanimonas Tolumonas Zobellella"
family_to_genus[Alcanivoracaceae]="Alcanivorax Marinicella Ketobacter"
family_to_genus[Halieaceae]="Halioglobus Parahaliea"
family_to_genus[Cellvibrionaceae]="Cellvibrio Marinimicrobium Teredinibacter Saccharophagus Simiduia Agaribacterium Marinagarivorans Gilvimarinus"
family_to_genus[Succinivibrionaceae]="Anaerobiospirillum Succinivibrio Ruminobacter Succinatimonas"
family_to_genus[Halomonadaceae]="Cobetia Halomonas  Chromohalobacter Salinicola Carnimonas Halovibrio Zymobacter Halotalea Kushneria"
family_to_genus[Coxiellaceae]="Coxiella Aquicella"
family_to_genus[Thiotrichaceae]="Thiothrix Leucothrix"
family_to_genus[Chromatiaceae]="Thiocapsa Rheinheimera Nitrosococcus Allochromatium Thiocystis Chromatium Lamprocystis Imhoffiella"
family_to_genus[Shewanellaceae]="Shewanella"
family_to_genus[Yersiniaceae]="Serratia Yersinia Rahnella  Rouxiella Chania"
family_to_genus[Rhodanobacteraceae]="Rhodanobacter  Frateuria Dyella Luteibacter Dokdonella Fulvimonas Metallibacterium Rudaea Aquimonas"
family_to_genus[Spongiibacteraceae]="Spongiibacter  Zhongshania Oceanicoccus Dasania"
family_to_genus[Hafniaceae]="Hafnia Edwardsiella"
family_to_genus[Salinisphaeraceae]="Salinisphaera"
family_to_genus[Morganellaceae]="Proteus Providencia Photorhabdus Xenorhabdus Morganella Moellerella"
family_to_genus[Legionellaceae]="Legionella Tatlockia Fluoribacter"
family_to_genus[Vibrionaceae]="Vibrio Photobacterium Aliivibrio Salinivibrio Grimontia Enterovibrio Paraphotobacterium"
family_to_genus[Sinobacteraceae]="Hydrocarboniphaga Polycyclovorans Solimonas Steroidobacter Sinimarinibacterium Fontimonas"
family_to_genus[Pasteurellaceae]="Haemophilus Pasteurella Aggregatibacter Gallibacterium Actinobacillus Basfia Histophilus Glaesserella Rodentibacter  Mannheimia Bibersteinia Avibacterium Phocoenobacter Necropsobacter Caviibacterium Chelonobacter Muribacter"
family_to_genus[Pectobacteriaceae]="Pectobacterium Sodalis Dickeya  Lonsdalea Brenneria"
family_to_genus[Hahellaceae]="Hahella Zooshikella"
family_to_genus[Kangiellaceae]="Kangiella Pleionea"
family_to_genus[Ferrimonadaceae]="Ferrimonas Paraferrimonas"
family_to_genus[Halothiobacillaceae]="Halothiobacillus"
family_to_genus[Colwelliaceae]="Colwellia Thalassotalea Thalassomonas"
family_to_genus[Ectothiorhodospiraceae]="Alkalilimnicola Thioalkalivibrio Halorhodospira Oceanococcus Spiribacter Nitrococcus Thiorhodospira Ectothiorhodospira Aquisalimonas Thiohalomonas"
family_to_genus[Cardiobacteriaceae]="Dichelobacter Cardiobacterium Suttonella"
family_to_genus[Psychromonadaceae]="Psychromonas"
family_to_genus[Orbaceae]="Gilliamella Frischella"
family_to_genus[Oleiphilaceae]=""
family_to_genus[Microbulbiferaceae]="Microbulbifer"
family_to_genus[Moritellaceae]="Moritella"
family_to_genus[Granulosicoccaceae]="Granulosicoccus"
family_to_genus[Wenzhouxiangellaceae]="Wenzhouxiangella"
family_to_genus[Thiolinaceae]="Thiolinea"
family_to_genus[Endozoicomonadaceae]="Endozoicomonas Parendozoicomonas"
family_to_genus[Saccharospirillaceae]="Reinekea Saccharospirillum"
family_to_genus[Budviciaceae]="Leminorella Pragia"
family_to_genus[Algiphilaceae]="Algiphilus"
family_to_genus[Ventosimonadaceae]="Ventosimonas"
family_to_genus[unclassified Thiotrichales]="Thioflexothrix"
family_to_genus[Acidiferrobacteraceae]="Acidiferrobacter Sulfurifustis"
family_to_genus[Balneatrichaceae]="Balneatrix"
family_to_genus[Thiofilaceae]="Thiofilum"
family_to_genus[Fastidiosibacteraceae]="Caedibacter Cysteiniphilum"
family_to_genus[Crenotrichaceae]="Crenothrix"
family_to_genus[Immundisolibacteraceae]="Immundisolibacter"


##loop through all order-level models
for filename in models/Order/*; ##TODO location aendern
do 
	IFS='.'
	read -ra ADD <<< "$(basename "$filename")"
	IFS=' '
	$gkmpredict_location $input_file $filename $tmp_folder$ADD$predict_suffix
	printf '%s %s\n' "$ADD" "$(awk '{print $2}' $tmp_folder$ADD$predict_suffix)" >> $col_order
	rm $tmp_folder$ADD$predict_suffix
done
##Sort the results; get top3 orders
sort -V -k2 -o $col_order $col_order
top_three_orders="$(awk 'NR==1, NR==3{print $1}' $col_order)"


##loop through the relevant family-level models
while read -r chosen_order; do
	these_families=${order_to_family[$chosen_order]}
	if test -z "$these_families"; 
		then
			echo "Skipping family for "$chosen_order
		else
			for family in $these_families
			do	
				$gkmpredict_location $input_file $family_model_folder$family$model_suffix $tmp_folder$family$predict_suffix
				printf '%s %s\n' "$family" "$(awk '{print $2}' $tmp_folder$family$predict_suffix)" >> $col_fam
				rm $tmp_folder$family$predict_suffix
			done;
	fi;
done <<< "$top_three_orders"
##Sort the results; get top3 families
sort -V -k2 -o $col_fam $col_fam
top_three_families="$(awk 'NR==1, NR==3{print $1}' $col_fam)"


##loop through the relevant genus-level models
while read -r chosen_family; do
	these_genii=${family_to_genus[$chosen_family]}
	if test -z "$these_genii"; 
		then
			echo "Skipping family for "$chosen_family
		else
			for genus in $these_genii
			do	
				$gkmpredict_location $input_file $genus_model_folder$genus$model_suffix $tmp_folder$genus$predict_suffix
				printf '%s %s\n' "$genus" "$(awk '{print $2}' $tmp_folder$genus$predict_suffix)" >> $col_genus
				rm $tmp_folder$genus$predict_suffix
			done;
	fi;
done <<< "$top_three_families"
##Sort the results; get top3 genii
sort -V -k2 -o $col_genus $col_genus
top_three_genii="$(awk 'NR==1, NR==3{print $1}' $col_genus)"
rm -r $tmp_folder

printf '%s\n%s\n%s\n%s\n%s\n%s' "Genus" "$(awk 'NR==1, NR==3' $col_genus)" "Family" "$(awk 'NR==1, NR==3' $col_fam)" "Order" "$(awk 'NR==1, NR==3' $col_order)" >> "${target_file}"
#TODO target_file löschen
##TODO laufen lassen auf BOriS-Rechner
